from gaussian_splatting.camera import build_camera
from gaussian_splatting.dataset.colmap import read_colmap_cameras, ColmapCameraDataset
from .dataset import RescaleCameraDatasetIface, RescaleTrainableCameraDataset


class RescaleColmapCameraDataset(RescaleCameraDatasetIface, ColmapCameraDataset):
    def __init__(self, colmap_folder, load_depth=False, rescale_factor=1.0):
        # https://github.com/yindaheng98/gaussian-splatting/blob/56576b647d9c5bd05300f5640cd03a8c75a760bc/gaussian_splatting/dataset/colmap/dataset.py#L89
        self.raw_cameras = read_colmap_cameras(colmap_folder, load_depth=load_depth)
        self.raw_image_hw = [(raw_camera.image_height, raw_camera.image_width) for raw_camera in self.raw_cameras]
        self.raw_cameras = [
            raw_camera._replace(
                image_height=round(h*rescale_factor), image_width=round(w*rescale_factor),
            ) for (h, w), raw_camera in zip(self.raw_image_hw, self.raw_cameras)]
        self.cameras = [build_camera(**cam._asdict(), custom_data=dict(fullimage_width=w, fullimage_height=h)) for (h, w), cam in zip(self.raw_image_hw, self.raw_cameras)]

    def to(self, device):
        # https://github.com/yindaheng98/gaussian-splatting/blob/56576b647d9c5bd05300f5640cd03a8c75a760bc/gaussian_splatting/dataset/colmap/dataset.py#L94
        self.cameras = [build_camera(**cam._asdict(), custom_data=dict(fullimage_width=w, fullimage_height=h), device=device) for (h, w), cam in zip(self.raw_image_hw, self.raw_cameras)]
        return self


def RescaleColmapTrainableCameraDataset(colmap_folder, load_depth=False, rescale_factor=1.0):
    # https://github.com/yindaheng98/gaussian-splatting/blob/56576b647d9c5bd05300f5640cd03a8c75a760bc/gaussian_splatting/dataset/colmap/dataset.py#L105
    return RescaleTrainableCameraDataset(RescaleColmapCameraDataset(colmap_folder, load_depth=load_depth, rescale_factor=rescale_factor))



from typing import List
from gaussian_splatting import Camera, GaussianModel
from gaussian_splatting.dataset import TrainableCameraDataset
from gaussian_splatting.trainer import DepthTrainerWrapper, NoopDensifier
from lapisgs.trainer import PartialDensificationTrainerWrapper
from reduced_3dgs.importance import ImportancePruner

# Importance Pruner


def PartialImportancePrunerInDensifyTrainer(
        model: GaussianModel,
        scene_extent: float,
        dataset: List[Camera],
        *args,
        importance_prune_from_iter=15000,
        importance_prune_until_iter=20000,
        importance_prune_interval: int = 1000,
        importance_score_resize=None,
        importance_prune_type="comprehensive",
        importance_prune_percent=0.1,
        importance_prune_thr_important_score=None,
        importance_prune_thr_v_important_score=3.0,
        importance_prune_thr_max_v_important_score=None,
        importance_prune_thr_count=1,
        importance_prune_thr_T_alpha=1.0,
        importance_prune_thr_T_alpha_avg=0.001,
        importance_v_pow=0.1,
        **kwargs):
    return PartialDensificationTrainerWrapper(
        lambda model, scene_extent: ImportancePruner(
            NoopDensifier(model),
            dataset,
            importance_prune_from_iter=importance_prune_from_iter,
            importance_prune_until_iter=importance_prune_until_iter,
            importance_prune_interval=importance_prune_interval,
            importance_score_resize=importance_score_resize,
            importance_prune_type=importance_prune_type,
            importance_prune_percent=importance_prune_percent,
            importance_prune_thr_important_score=importance_prune_thr_important_score,
            importance_prune_thr_v_important_score=importance_prune_thr_v_important_score,
            importance_prune_thr_max_v_important_score=importance_prune_thr_max_v_important_score,
            importance_prune_thr_count=importance_prune_thr_count,
            importance_prune_thr_T_alpha=importance_prune_thr_T_alpha,
            importance_prune_thr_T_alpha_avg=importance_prune_thr_T_alpha_avg,
            importance_v_pow=importance_v_pow,
        ),
        model,
        scene_extent,
        *args, **kwargs
    )


# Depth trainer
def DepthPartialImportancePrunerInDensifyTrainer(model: GaussianModel, scene_extent: float, dataset: TrainableCameraDataset, *args, **kwargs):
    return DepthTrainerWrapper(
        PartialImportancePrunerInDensifyTrainer,
        model, scene_extent, dataset,
        *args, **kwargs)


LapisImportanceTrainer = DepthPartialImportancePrunerInDensifyTrainer
BaseLapisImportanceTrainer = PartialImportancePrunerInDensifyTrainer



from typing import Callable, List
from gaussian_splatting import Camera, GaussianModel
from gaussian_splatting.dataset import TrainableCameraDataset
from gaussian_splatting.trainer import AbstractDensifier, DepthTrainerWrapper, NoopDensifier
from lapisgs.trainer import SplitClonePartialDensifierTrainerWrapper
from reduced_3dgs.pruning import BasePruner

# Reduced 3DGS


def PartialPrunerInDensifyTrainerWrapper(
        noargs_base_densifier_constructor: Callable[[GaussianModel, float, List[Camera]], AbstractDensifier],
        model: GaussianModel,
        scene_extent: float,
        dataset: List[Camera],
        *args,
        prune_from_iter=1000,
        prune_until_iter=15000,
        prune_interval: int = 100,
        box_size=1.,
        lambda_mercy=1.,
        mercy_minimum=3,
        mercy_type='redundancy_opacity',
        **kwargs):
    return SplitClonePartialDensifierTrainerWrapper(
        lambda model, scene_extent: BasePruner(
            noargs_base_densifier_constructor(model, scene_extent, dataset),
            scene_extent, dataset,
            prune_from_iter=prune_from_iter,
            prune_until_iter=prune_until_iter,
            prune_interval=prune_interval,
            box_size=box_size,
            lambda_mercy=lambda_mercy,
            mercy_minimum=mercy_minimum,
            mercy_type=mercy_type,
        ),
        model,
        scene_extent,
        *args, **kwargs
    )


def PartialPrunerInDensifyTrainer(
        model: GaussianModel,
        scene_extent: float,
        dataset: List[Camera],
        *args, **kwargs):
    return PartialPrunerInDensifyTrainerWrapper(
        lambda model, scene_extent, dataset: NoopDensifier(model),
        model, scene_extent, dataset,
        *args, **kwargs
    )


# Depth trainer

def DepthPrunerInDensifyTrainer(model: GaussianModel, scene_extent: float, dataset: TrainableCameraDataset, *args, **kwargs):
    return DepthTrainerWrapper(
        PartialPrunerInDensifyTrainer,
        model, scene_extent, dataset,
        *args, **kwargs)


LapisReducedTrainer = DepthPrunerInDensifyTrainer
BaseLapisReducedTrainer = PartialPrunerInDensifyTrainer
